@@ -29,10 +29,10 @@ import os.path
 
 import numpy as np
 
-from prody.atomic import Atomic, Atom, AtomGroup
+from prody.atomic import AtomGroup
 from prody.atomic import flags
 from prody.atomic import ATOMIC_FIELDS
-from prody.utilities import openFile, Everything
+from prody.utilities import openFile
 from prody import LOGGER, SETTINGS
 
 from .header import getHeaderDict, buildBiomolecules, assignSecstr
@@ -193,7 +193,13 @@ def parsePDBStream(stream, **kwargs):
     hd = None
     if model != 0:
         LOGGER.timeit()
-        lines = stream.readlines()
+        try:
+            lines = stream.readlines()
+        except AttributeError as err:
+            try:
+                lines = stream.read().split('\n')
+            except AttributeError:
+                raise err
         if not len(lines):
             raise ValueError('empty PDB file or stream')
         if header or biomol or secondary:
@@ -342,6 +348,7 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
     resnums = np.zeros(asize, dtype=ATOMIC_FIELDS['resnum'].dtype)
     chainids = np.zeros(asize, dtype=ATOMIC_FIELDS['chain'].dtype)
     hetero = np.zeros(asize, dtype=bool)
+    termini = np.zeros(asize, dtype=bool)
     altlocs = np.zeros(asize, dtype=ATOMIC_FIELDS['altloc'].dtype)
     icodes = np.zeros(asize, dtype=ATOMIC_FIELDS['icode'].dtype)
     serials = np.zeros(asize, dtype=ATOMIC_FIELDS['serial'].dtype)
@@ -350,7 +357,6 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
         elements = np.zeros(asize, dtype=ATOMIC_FIELDS['element'].dtype)
         bfactors = np.zeros(asize, dtype=ATOMIC_FIELDS['beta'].dtype)
         occupancies = np.zeros(asize, dtype=ATOMIC_FIELDS['occupancy'].dtype)
-        secondary = None
         anisou = None
         siguij = None
     else:
@@ -493,6 +499,7 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
                 chainids = np.concatenate((chainids,
                     np.zeros(asize, ATOMIC_FIELDS['chain'].dtype)))
                 hetero = np.concatenate((hetero, np.zeros(asize, bool)))
+                termini = np.concatenate((termini, np.zeros(asize, bool)))
                 altlocs = np.concatenate((altlocs,
                     np.zeros(asize, ATOMIC_FIELDS['altloc'].dtype)))
                 icodes = np.concatenate((icodes,
@@ -522,6 +529,9 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
         #elif startswith == 'END   ' or startswith == 'CONECT':
         #    i += 1
         #    break
+        elif not onlycoords and (startswith == 'TER   ' or
+            startswith.strip() == 'TER'):
+            termini[acount - 1] = True
         elif startswith == 'ENDMDL' or startswith[:3] == 'END':
             if acount == 0:
                 # If there is no atom record between ENDMDL & END skip to next
@@ -550,37 +560,56 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
                                     'same number of atoms')
                 # this is where to decide if more coordsets should be expected
                 if END:
+                    coordinates.resize((acount, 3))
                     if addcoords:
-                        atomgroup.addCoordset(coordinates[:acount])
+                        atomgroup.addCoordset(coordinates)
                     else:
-                        atomgroup._setCoords(coordinates[:acount])
+                        atomgroup._setCoords(coordinates)
                 else:
                     coordsets = np.zeros((diff/acount+1, acount, 3))
                     coordsets[0] = coordinates[:acount]
                     onlycoords = True
+                atomnames.resize(acount)
+                resnames.resize(acount)
+                resnums.resize(acount)
+                chainids.resize(acount)
+                hetero.resize(acount)
+                termini.resize(acount)
+                altlocs.resize(acount)
+                icodes.resize(acount)
+                serials.resize(acount)
                 if not only_subset:
-                    atomnames = np.char.strip(atomnames[:acount])
-                    resnames = np.char.strip(resnames[:acount])
-                atomgroup.setNames(atomnames[:acount])
-                atomgroup.setResnames(resnames[:acount])
-                atomgroup.setResnums(resnums[:acount])
-                atomgroup.setChids(chainids[:acount])
-                atomgroup.setFlags('hetatm', hetero[:acount])
-                atomgroup.setAltlocs(altlocs[:acount])
-                atomgroup.setIcodes(np.char.strip(icodes[:acount]))
-                atomgroup.setSerials(serials[:acount])
+                    atomnames = np.char.strip(atomnames)
+                    resnames = np.char.strip(resnames)
+                atomgroup.setNames(atomnames)
+                atomgroup.setResnames(resnames)
+                atomgroup.setResnums(resnums)
+                atomgroup.setChids(chainids)
+                atomgroup.setFlags('hetatm', hetero)
+                atomgroup.setFlags('pdbter', termini)
+                atomgroup.setAltlocs(altlocs)
+                atomgroup.setIcodes(np.char.strip(icodes))
+                atomgroup.setSerials(serials)
                 if isPDB:
-                    atomgroup.setBetas(bfactors[:acount])
-                    atomgroup.setOccupancies(occupancies[:acount])
-                    atomgroup.setSegnames(np.char.strip(segnames[:acount]))
-                    atomgroup.setElements(np.char.strip(elements[:acount]))
+                    bfactors.resize(acount)
+                    occupancies.resize(acount)
+                    segnames.resize(acount)
+                    elements.resize(acount)
+                    atomgroup.setBetas(bfactors)
+                    atomgroup.setOccupancies(occupancies)
+                    atomgroup.setSegnames(np.char.strip(segnames))
+                    atomgroup.setElements(np.char.strip(elements))
                     if anisou is not None:
-                        atomgroup.setAnisous(anisou[:acount] / 10000)
+                        anisou.resize((acount, 6))
+                        atomgroup.setAnisous(anisou / 10000)
                     if siguij is not None:
-                        atomgroup.setAnistds(siguij[:acount] / 10000)
+                        siguij.resize((acount, 6))
+                        atomgroup.setAnistds(siguij / 10000)
                 else:
-                    atomgroup.setCharges(charges[:acount])
-                    atomgroup.setRadii(radii[:acount])
+                    charges.resize(acount)
+                    radii.resize(acount)
+                    atomgroup.setCharges(charges)
+                    atomgroup.setRadii(radii)
 
                 nmodel += 1
                 n_atoms = acount
@@ -630,46 +659,60 @@ def _parsePDBLines(atomgroup, lines, split, model, chain, subset,
         if acount == atomgroup.numAtoms():
             coordsets[nmodel] = coordinates
             nmodel += 1
-        if nmodel == coordsets.shape[0]:
-            if addcoords:
-                atomgroup.addCoordset(coordsets)
-            else:
-                atomgroup._setCoords(coordsets)
-        else:
-            if addcoords:
-                atomgroup.addCoordset(coordsets[:nmodel])
-            else:
-                atomgroup._setCoords(coordsets[:nmodel])
-    elif not END:
-        # this means last line wast an ATOM line, so atomgroup is not decorated
+        del coordinates
+        coordsets.resize((nmodel, atomgroup.numAtoms(), 3))
         if addcoords:
-            atomgroup.addCoordset(coordinates[:acount])
+            atomgroup.addCoordset(coordsets)
         else:
-            atomgroup._setCoords(coordinates[:acount])
+            atomgroup._setCoords(coordsets)
+    elif not END:
+        # this means last line was an ATOM line, so atomgroup is not decorated
+        coordinates.resize((acount, 3))
+        if addcoords:
+            atomgroup.addCoordset(coordinates)
+        else:
+            atomgroup._setCoords(coordinates)
+        atomnames.resize(acount)
+        resnames.resize(acount)
+        resnums.resize(acount)
+        chainids.resize(acount)
+        hetero.resize(acount)
+        termini.resize(acount)
+        altlocs.resize(acount)
+        icodes.resize(acount)
+        serials.resize(acount)
         if not only_subset:
-            atomnames = np.char.strip(atomnames[:acount])
-            resnames = np.char.strip(resnames[:acount])
-        atomgroup.setNames(atomnames[:acount])
-        atomgroup.setResnames(resnames[:acount])
-        atomgroup.setResnums(resnums[:acount])
-        atomgroup.setChids(chainids[:acount])
-        atomgroup.setFlags('hetatm', hetero[:acount])
-        atomgroup.setAltlocs(altlocs[:acount])
-        atomgroup.setIcodes(np.char.strip(icodes[:acount]))
-        atomgroup.setSerials(serials[:acount])
+            atomnames = np.char.strip(atomnames)
+            resnames = np.char.strip(resnames)
+        atomgroup.setNames(atomnames)
+        atomgroup.setResnames(resnames)
+        atomgroup.setResnums(resnums)
+        atomgroup.setChids(chainids)
+        atomgroup.setFlags('hetatm', hetero)
+        atomgroup.setFlags('pdbter', termini)
+        atomgroup.setAltlocs(altlocs)
+        atomgroup.setIcodes(np.char.strip(icodes))
+        atomgroup.setSerials(serials)
         if isPDB:
             if anisou is not None:
-                atomgroup.setAnisous(anisou[:acount] / 10000)
+                anisou.resize((acount, 6))
+                atomgroup.setAnisous(anisou / 10000)
             if siguij is not None:
-                atomgroup.setAnistds(siguij[:acount] / 10000)
-            atomgroup.setSegnames(np.char.strip(segnames[:acount]))
-            atomgroup.setElements(np.char.strip(elements[:acount]))
-            atomgroup.setBetas(bfactors[:acount])
-            atomgroup.setOccupancies(occupancies[:acount])
+                siguij.resize((acount, 6))
+                atomgroup.setAnistds(siguij / 10000)
+            bfactors.resize(acount)
+            occupancies.resize(acount)
+            segnames.resize(acount)
+            elements.resize(acount)
+            atomgroup.setSegnames(np.char.strip(segnames))
+            atomgroup.setElements(np.char.strip(elements))
+            atomgroup.setBetas(bfactors)
+            atomgroup.setOccupancies(occupancies)
         else:
-            atomgroup.setCharges(charges[:acount])
-            atomgroup.setRadii(radii[:acount])
-
+            charges.resize(acount)
+            radii.resize(acount)
+            atomgroup.setCharges(charges)
+            atomgroup.setRadii(radii)
 
     if altloc and altloc_torf:
         _evalAltlocs(atomgroup, altloc, chainids, resnums, resnames, atomnames)
